import { Machine, State, On, SideEffect, Goto, Spawn, Unspawn, Interpreter, Interpreter_T, send, Event_T, Context } from '../index';

const wait = (ms:number)=>new Promise((resolve)=>{ setTimeout(()=>{ resolve(1); }, ms); });
const makeRequest = (ctx,e,self)=>{ send(ctx.serverActor, ['received-request',self]); };
const sendResponse = (ctx,e,self)=>{ send(ctx.clientActor, ['received-response',self]); };
const startTimer = async (ctx,e,self)=>{ await wait(1500); send(self, ['timer-finished',null]); }
const log = (ctx, e, self:Interpreter_T)=>{ console.log(self.state, ctx); };

const client = 
  Machine(
    State('idle',
      On('entry',
        SideEffect(log),
      ),
      On('server-created',
        SideEffect((_ctx,[_eventName,serverActor],self)=>{ self.context.serverActor=serverActor; }),
        Goto('making-request')
      )
    ),
    State('making-request',
      On('entry', 
        SideEffect(log),
        SideEffect(makeRequest),
        Context((ctx)=>({...ctx, requestsMade: ctx.requestsMade+1})),
        Goto('awaiting-response')
      ),
    ),
    State('awaiting-response',
      On('entry',
        SideEffect(log),
      ),
      On('received-response', 
        SideEffect(log),
        Context((ctx)=>({...ctx, responsesReceived: ctx.responsesReceived+1})),
        Goto('making-request')
      ),
    ),
  );

const server = 
  Machine(
    State('awaiting-request',
      On('entry',
        SideEffect(log),
      ),
      On('received-request', 
        SideEffect((_ctx,[_eventName,clientActor],self)=>{ self.context.clientActor=clientActor; }),
        Goto('sending-response')
      ),
    ),
    State('sending-response',
      On('entry', 
        SideEffect(log),
        SideEffect(startTimer)
      ),
      On('timer-finished',
        SideEffect(sendResponse),
        Goto('awaiting-request')
      )
    ),
  );

const clientActor = Interpreter(client, {context:{requestsMade:0, responsesReceived:0}});
const serverActor = Interpreter(server, {context:{}});
send(clientActor, ['server-created', serverActor]);